@@ -104,6 +104,29 @@ agent_Copy(const agent_p agent)
   return cp_agent;
 }
 
+agent_p
+agent_New(void)
+{
+  agent_p agent;
+  agent = (agent_p)malloc(sizeof(agent_t));
+  if(agent==NULL) {
+    fprintf(stderr, "Memory error at %s:%d\n",
+        __FILE__, __LINE__);
+    return NULL;
+  }
+  /* Just init everything to zero */
+  memset(agent, 0, sizeof(agent_t));
+
+  /* Lets go ahead and allocate the threading stuff */
+  MUTEX_NEW(agent->run_lock);
+  MUTEX_INIT(agent->run_lock);
+
+  MUTEX_NEW(agent->lock);
+  MUTEX_INIT(agent->lock);
+
+  return agent;
+}
+
   agent_p 
 agent_Initialize(
     struct mc_platform_s *mc_platform,
@@ -144,6 +167,9 @@ agent_Initialize(
   agent->agent_thread = NULL;
   agent->agent_thread_id = 0;
 
+  /* Set up an empty mailbox */
+  agent->mailbox = agent_mailbox_New();
+
   /* parse the xml */
   agent->datastate = agent_datastate_New();
   agent->datastate->xml_agent_root = message->xml_payload;
@@ -235,6 +261,7 @@ agent_Destroy(agent_p agent)
   agent_datastate_Destroy(agent->datastate);
   free(agent->agent_thread);
   free(agent->run_lock);
+  agent_mailbox_Destroy(agent->mailbox);
   /* deallocate the agent */
   free(agent);
   agent = NULL;
@@ -429,6 +456,68 @@ agent_RunChScriptThread(void* ChAgent)
       );
 
   /* Following are the declarations of the agent-space api functions. */
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "int mc_AclDestroy(void* acl_message);",
+      (ChFuncdl_t)MC_AclDestroy_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "void* mc_AclNew(void);",
+      (ChFuncdl_t)MC_AclNew_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "void* mc_AclPost(void* agent, void* acl_message);",
+      (ChFuncdl_t)MC_AclPost_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "void* mc_AclRetrieve(void* agent);",
+      (ChFuncdl_t)MC_AclRetrieve_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "void* mc_AclReply(void* acl_message);",
+      (ChFuncdl_t)MC_AclReply_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "int mc_AclSend(void* acl_message);",
+      (ChFuncdl_t)MC_AclSend_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "void* mc_AclWaitRetrieve(void* agent);",
+      (ChFuncdl_t)MC_AclWaitRetrieve_chdl
+      );
+  /* begin Acl Helper Functions */
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "int mc_Acl_SetPerformative(void* acl_message, int performative);",
+      (ChFuncdl_t)MC_AclSetPerformative_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "int mc_Acl_SetSender(void* acl_message, char* name, char* address);",
+      (ChFuncdl_t)MC_AclSetSender_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "int mc_Acl_AddReceiver(void* acl_message, char* name, char* address);",
+      (ChFuncdl_t)MC_AclAddReceiver_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "int mc_Acl_AddReplyTo(void* acl_message, char* name, char* address);",
+      (ChFuncdl_t)MC_AclAddReplyTo_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "int mc_Acl_SetContent(void* acl_message, char* content);",
+      (ChFuncdl_t)MC_AclSetContent_chdl
+      );
+  /* end Acl Helper Functions */
   Ch_DeclareFunc(
       agent->agent_interp,
       "int mc_AddAgent(void* agent);",
@@ -456,7 +545,12 @@ agent_RunChScriptThread(void* ChAgent)
       );
   Ch_DeclareFunc(
       agent->agent_interp,
-      "int mc_CondBroadCast(int id);",
+      "MCAgent_t mc_ComposeAgent(const char* name, *home, *owner, *code, *return_var_name, *server, int persistent);",
+      (ChFuncdl_t)MC_ComposeAgent_chdl
+      );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "int mc_CondBroadcast(int id);",
       (ChFuncdl_t)MC_CondBroadcast_chdl
       );
   Ch_DeclareFunc(
@@ -484,6 +578,11 @@ agent_RunChScriptThread(void* ChAgent)
       "int mc_DeregisterService(int agentID, char* serviceName);",
       (ChFuncdl_t)MC_DeregisterService_chdl
       );
+  Ch_DeclareFunc(
+      agent->agent_interp,
+      "int mc_DestroyServiceSearchResult( char** agentName, char** serviceName, int* agentID, int numResult);",
+      (ChFuncdl_t)MC_DestroyServiceSearchResult_chdl 
+      );
   Ch_DeclareFunc(
       agent->agent_interp,
       "int mc_End(void);",
@@ -685,8 +784,7 @@ FIXME: This is silly
     tmpnam(temp_store_file);
 #endif
     TEMP_FILE = fopen(temp_store_file, "w");
-
-
+    
     /* write the data to a ch-file */
     n = fwrite(
         (void *)agent->datastate->agent_code, 
@@ -726,6 +824,8 @@ FIXME: This is silly
     agent->datastate->tasks[progress]->agent_return_data = 
       result;
   } else {
+    agent_return_data_Destroy(
+        agent->datastate->tasks[progress]->agent_return_data );
     agent->datastate->tasks[progress]->agent_return_data = NULL;
   }
 
