@@ -37,6 +37,8 @@
 #include "libmc.h"
 #include "message.h"
 #include "macros.h"
+#include "data_structures.h"
+#include "agent_mailbox.h"
 
 struct mc_platform_s;
 
@@ -79,6 +81,8 @@ struct agent_s {
 #endif
   THREAD_T *agent_thread;
 
+  /* Agent Mailbox */
+  agent_mailbox_p mailbox;
 
   /* Agent Flags */
   int agent_pipe_active;
@@ -98,6 +102,9 @@ typedef agent_t* MCAgent_t;
 typedef agent_t* agent_p;
 #endif
 
+agent_p
+agent_New(void);
+
 agent_p
 agent_Copy(const agent_p agent);
 
