@@ -31,6 +31,44 @@
 
 #ifndef _AGENT_LIB_H_
 #define _AGENT_LIB_H_
+EXPORTCH int
+MC_AclDestroy_chdl(void* varg);
+
+EXPORTCH void* 
+MC_AclNew_chdl(void* varg);
+
+EXPORTCH void*
+MC_AclPost_chdl(void* varg);
+
+EXPORTCH void*
+MC_AclReply_chdl(void* varg);
+
+EXPORTCH void*
+MC_AclRetrieve_chdl(void* varg);
+
+EXPORTCH int
+MC_AclSend_chdl(void* varg);
+
+EXPORTCH void*
+MC_AclWaitRetrieve_chdl(void *varg);
+
+/* BEGIN Acl helper functions */
+
+EXPORTCH void*
+MC_AclSetPerformative_chdl(void *varg);
+
+EXPORTCH void*
+MC_AclSetSender_chdl(void *varg);
+
+EXPORTCH void*
+MC_AclAddReceiver_chdl(void *varg);
+
+EXPORTCH void*
+MC_AclAddReplyTo_chdl(void *varg);
+
+EXPORTCH void*
+MC_AclSetContent_chdl(void *varg);
+/* END Acl helper functions */
 
 EXPORTCH int
 MC_AddAgent_chdl(void *varg);
@@ -47,6 +85,9 @@ MC_BarrierInit_chdl(void *varg);
 EXPORTCH int
 MC_CallAgentFunc_chdl(void *varg);
 
+EXPORTCH int
+MC_ComposeAgent_chdl(void *varg);
+
 EXPORTCH int
 MC_CondBroadcast_chdl(void *varg);
 
@@ -65,6 +106,9 @@ MC_DeleteAgent_chdl(void *varg);
 EXPORTCH int
 MC_DeregisterService_chdl(void *varg); 
 
+EXPORTCH int
+MC_DestroyServiceSearchResult_chdl(void* varg);
+
 EXPORTCH int
 MC_End_chdl(void *varg);
 
