@@ -57,6 +57,9 @@ typedef struct agent_task_s{
 
 typedef agent_task_t* agent_task_p;
 
+agent_task_p
+agent_task_New(void);
+
 agent_task_p 
 agent_task_Copy(agent_task_p task);
 
