@@ -43,28 +43,93 @@
 
 /* Filename: libmc.h */
 /* Current functions implemented 
-   MC_AddAgent
-   MC_ChInitializeOptions
-   MC_FindAgentByName 
-   MC_GetAgentExecEngine
-   MC_GetAgentStatus
-   MCHandle_sInit
-   MC_SetThreadOn
-   MC_PrintAgentCode
-   MC_RetrieveAgentCode
-   MC_SendACLMessageFile
-   MC_SetAgentStatus
-   MC_TerminateAgent
-   MC_Wait 
-   MC_WaitAgent
-   MC_WaitRetrieveAgent
-   MC_WaitSignal
+MC_AclDestroy
+MC_AclNew
+MC_AclPost
+MC_AclReply
+MC_AclRetrieve
+MC_AclSend
+MC_AclWaitRetrieve
+MC_AddAgent 
+MC_Barrier 
+MC_BarrierInit 
+MC_BarrierDelete 
+MC_CallAgentFunc
+MC_ChInitializeOptions 
+MC_ComposeAgent
+MC_CondBroadcast 
+MC_CondSignal 
+MC_CondWait  
+MC_CondReset 
+MC_CopyAgent 
+MC_DeleteAgent 
+MC_DeregisterService
+MC_End 
+MC_FindAgentByName
+MC_FindAgentByID
+MC_GetAgentArrivalTime 
+MC_GetAgentStatus 
+MC_GetAgentXMLString 
+MC_GetAgentExecEngine 
+MC_GetAgentID
+MC_GetAgentName
+MC_GetAgentReturnData
+MC_GetAgentNumTasks 
+MC_GetAgentType 
+MC_GetAllAgents 
+MC_HaltAgency 
+MC_Initialize
+MC_InitializeAgencyOptions 
+MC_MutexLock 
+MC_MutexUnlock 
+MC_PrintAgentCode 
+MC_RegisterService
+MC_ResumeAgency 
+MC_RetrieveAgent 
+MC_RetrieveAgentCode 
+MC_ResetSignal 
+MC_SearchForService
+MC_SemaphorePost 
+MC_SemaphoreWait 
+MC_SendCh
+MC_SendAgentMigrationMessage
+MC_SendAgentMigrationMessageFile
+MC_SendSteerCommand 
+MC_SetAgentStatus 
+MC_SetDefaultAgentStatus
+MC_SetThreadOn 
+MC_SetThreadsAllOn
+MC_SetThreadOff 
+MC_SetThreadsAllOff
+MC_Steer
+MC_SteerControl 
+MC_SyncDelete 
+MC_SyncInit 
+MC_TerminateAgent 
+MC_MainLoop 
+MC_MainLoop
+MC_WaitAgent 
+MC_WaitRetrieveAgent 
+MC_WaitSignal 
 
    *** */
 
 #ifndef _MOBILEC_H_
 #define _MOBILEC_H_
 
+#ifdef WIN32
+	#ifdef _MC_DLL
+		// Building a .dll - export functions as marked
+		#define EXPORTMC _declspec(dllexport)
+	#else
+		// Not building a .dll - building .lib assumed - no export required
+		#define EXPORTMC
+	#endif
+#else
+	// Not windows
+	#define EXPORTMC
+#endif
+
 /* The MC_Wait() function is deprecated as of version 1.9.4.
  * All calls to MC_Wait() should now be changed to MC_MainLoop() */
 #define MC_Wait(arg1) \
@@ -176,13 +241,14 @@ typedef agent_t* MCAgent_t;
 typedef agent_t* agent_p;
 #endif
 
-enum MC_AgentType_e{ MC_REMOTE_AGENT, MC_LOCAL_AGENT, MC_RETURN_AGENT };
+enum MC_AgentType_e{ MC_NONE = -1, MC_REMOTE_AGENT = 0, MC_LOCAL_AGENT, MC_RETURN_AGENT };
 
 /**
  * \brief         An agent's current execution state
  */
 enum MC_AgentStatus_e{ 
-  MC_WAIT_CH,        /*!< Waiting to be started */
+	MC_NO_STATUS = -1,
+    MC_WAIT_CH = 0,    /*!< Waiting to be started */
     MC_WAIT_MESSGSEND, /*!< Finished, waiting to migrate */
     MC_AGENT_ACTIVE,   /*!< Running */
     MC_AGENT_NEUTRAL,  /*!< Not running, but do not flush */
@@ -190,6 +256,105 @@ enum MC_AgentStatus_e{
     MC_WAIT_FINISHED   /*!< Finished, waiting to be flushed */
 };
 
+struct fipa_acl_message_s;
+
+/**
+ * \brief         Destroy a FIPA ACL message
+ *
+ * \param message The ACL message to destroy
+ *
+ * \return        0 on success, error code on failure.
+ */
+EXPORTMC int 
+MC_AclDestroy(struct fipa_acl_message_s* message);
+
+/**
+ * \brief         Allocate a new ACL Message
+ *
+ * \return        A newly allocated and empty ACL message.
+ */
+EXPORTMC extern struct fipa_acl_message_s*
+MC_AclNew(void);
+
+/**
+ * \brief         Post ACL message to agent
+ *
+ * \param agent   The agent to post the message to
+ * \param message The message to post
+ *
+ * \return        0 if successful, or error_code_t type.
+ */
+EXPORTMC extern int MC_AclPost(MCAgent_t agent, struct fipa_acl_message_s* message);
+
+/**
+ * \brief         Reply to an ACL message
+ *
+ * \param acl_message The incoming acl message to reply to
+ *
+ * \return        A newly allocated ACL message
+ *
+ * \note          This function simply generates a new ACL message with the 'receiver' field
+ *                automatically set to the 'sender' field of the incoming message.
+ */
+EXPORTMC extern struct fipa_acl_message_s*
+MC_AclReply(struct fipa_acl_message_s* acl_message);
+
+/**
+ * \brief         Retrieve an ACL message
+ *
+ * \param agent   Agent to retrieve message from.
+ * 
+ * \return        an ACL message struct on success or NULL on failure
+ */
+EXPORTMC extern struct fipa_acl_message_s* MC_AclRetrieve(MCAgent_t agent);
+
+/**
+ * \brief         Send a composed ACL Message
+ *
+ * \param attr    An initialized and running MobileC agency
+ * \param acl     An allocated and fully composed ACL message.
+ *
+ * \return        0 if successful, error code on failure.
+ */
+EXPORTMC extern int MC_AclSend(MCAgency_t attr, struct fipa_acl_message_s* acl);
+
+/**
+ * \brief         Wait for and retrieve an ACL message
+ *
+ * \param agent   Agent to retrieve message from.
+ *
+ * \return        an ACL message struct on success or NULL on failure
+ */
+EXPORTMC extern struct fipa_acl_message_s* MC_AclWaitRetrieve(MCAgent_t agent);
+
+/* ACL Helper Functions Here */
+enum fipa_performative_e;
+EXPORTMC int MC_AclSetPerformative(
+    struct fipa_acl_message_s* acl,
+    enum fipa_performative_e performative );
+
+EXPORTMC int MC_AclSetSender(
+    struct fipa_acl_message_s* acl,
+    const char* name,
+    const char* address );
+  
+EXPORTMC int MC_AclAddReceiver(
+    struct fipa_acl_message_s* acl,
+    const char* name,
+    const char* address );
+
+EXPORTMC int MC_AclAddReplyTo(
+    struct fipa_acl_message_s* acl,
+    const char* name,
+    const char* address);
+
+EXPORTMC int MC_AclSetContent(
+    struct fipa_acl_message_s* acl,
+    const char* content );
+
+/* End ACL Helper Functions */
+
+
 /**
  * \brief         Add an agent to the agency 'attr'
  *
@@ -198,7 +363,7 @@ enum MC_AgentStatus_e{
  *
  * \return        0 if successful, or error_code_t type
  */
-extern int MC_AddAgent( 
+EXPORTMC extern int MC_AddAgent( 
     MCAgency_t attr, 
     MCAgent_t agent);
 
@@ -211,7 +376,7 @@ extern int MC_AddAgent(
  * \return          returns 0 on success, error if the node is not
  *                  found or other failure.
  */
-extern int MC_BarrierDelete(MCAgency_t attr, int id);
+EXPORTMC extern int MC_BarrierDelete(MCAgency_t attr, int id);
 
 /**
  * \brief           Initialize a MobileC Barrier 
@@ -228,7 +393,7 @@ extern int MC_BarrierDelete(MCAgency_t attr, int id);
  * The following example demonstrates an agent which sets up an MC_Barrier.
  * \include mc_barrier_example/comm_agent.xml
  */
-extern int MC_BarrierInit(MCAgency_t attr, int id, int num_procs);
+EXPORTMC extern int MC_BarrierInit(MCAgency_t attr, int id, int num_procs);
 
 /**
  * \brief           Use custom ChOptions_t type for internal Ch interpretor
@@ -238,7 +403,7 @@ extern int MC_BarrierInit(MCAgency_t attr, int id, int num_procs);
  *
  * \return          0 on success, error_code_t type on failure
  */
-extern int MC_ChInitializeOptions(MCAgency_t attr, ChOptions_t *options);
+EXPORTMC extern int MC_ChInitializeOptions(MCAgency_t attr, ChOptions_t *options);
 
 /**
  * \brief           Calls a function defined in an agent
@@ -256,13 +421,24 @@ extern int MC_ChInitializeOptions(MCAgency_t attr, ChOptions_t *options);
  * \section Example
  * \include persistent_example/host.c
  */
-extern int MC_CallAgentFunc(
+EXPORTMC extern int MC_CallAgentFunc(
         MCAgent_t agent,
         const char* funcName,
         void* returnVal,
         void* varg
         );
 
+MCAgent_t
+MC_ComposeAgent(
+    const char* name,
+    const char* home,
+    const char* owner, 
+    const char* code,
+    const char* return_var_name,
+    const char* server,
+    int persistent
+    );
+
 /**
  * \brief           Wakes up all agents/threads waiting on a condition variable
  *
@@ -273,7 +449,7 @@ extern int MC_CallAgentFunc(
  *
  * \return          0 on success, error_code_t type on failure
  */
-extern int MC_CondBroadcast(MCAgency_t attr, int id);
+EXPORTMC extern int MC_CondBroadcast(MCAgency_t attr, int id);
 
 /**
  * \brief           Wakes up at least one thread waiting on a condition variable
@@ -290,7 +466,7 @@ extern int MC_CondBroadcast(MCAgency_t attr, int id);
  * which is nearly identical to the binary space api function.
  * \include agent_cond_example/wake.xml
  */
-extern int MC_CondSignal(MCAgency_t attr, int id);
+EXPORTMC extern int MC_CondSignal(MCAgency_t attr, int id);
 
 /**
  * \brief           Reset a previously signalled MobileC condition variable
@@ -302,7 +478,7 @@ extern int MC_CondSignal(MCAgency_t attr, int id);
  *
  * \return          0 on success, error_code_t type on failure
  */
-extern int MC_CondReset(MCAgency_t attr, int id);
+EXPORTMC extern int MC_CondReset(MCAgency_t attr, int id);
 
 /**
  * \brief           Wait on a MobileC synchronization variable
@@ -317,7 +493,7 @@ extern int MC_CondReset(MCAgency_t attr, int id);
  * The following example demonstrates the agent-space version of this function.
  * \include agent_cond_example/sleep.xml
  */
-extern int MC_CondWait(MCAgency_t attr, int id);
+EXPORTMC extern int MC_CondWait(MCAgency_t attr, int id);
 
 /**
  * \brief           Performs a deep-copy of an agent structure
@@ -338,7 +514,7 @@ int MC_CopyAgent(MCAgent_t* agent_out, const MCAgent_t agent_in);
  * \return          0 on success, error_code_t type on failure
  *
  */
-extern int MC_DeleteAgent(MCAgent_t agent);
+EXPORTMC extern int MC_DeleteAgent(MCAgent_t agent);
 
 /**
  * \brief           End an agency
@@ -350,7 +526,24 @@ extern int MC_DeleteAgent(MCAgent_t agent);
  * \section Example
  * \include hello_world/client.c
  */
-extern int MC_End(MCAgency_t attr);
+EXPORTMC extern int MC_End(MCAgency_t attr);
+
+/**
+ * \brief           Free memory allocated by a Service Search operation
+ *
+ * \param agentName agent names returned by a search operation.
+ * \param serviceName service names return by a search operation.
+ * \param agentID   list of agent id's returned by a search operation.
+ * \param numResult The number of hits returned by a search operation.
+ *
+ * \return          0 on success, error code on failure.
+ */
+int MC_DestroyServiceSearchResult(
+    char** agentName,
+    char** serviceName,
+    int* agentID,
+    int numResult);
+
 
 /**
  * \brief           Find an agent by its name
@@ -362,7 +555,7 @@ extern int MC_End(MCAgency_t attr);
  * \section Example
  * \include multi_task_example/client.c
  */
-extern MCAgent_t MC_FindAgentByName(MCAgency_t attr, const char *name);
+EXPORTMC extern MCAgent_t MC_FindAgentByName(MCAgency_t attr, const char *name);
 
 /**
  * \brief           Find an agent by its id
@@ -372,7 +565,7 @@ extern MCAgent_t MC_FindAgentByName(MCAgency_t attr, const char *name);
  *
  * \return          a valid agent on success, NULL on failure
  */
-extern MCAgent_t MC_FindAgentByID(MCAgency_t attr, int ID);
+EXPORTMC extern MCAgent_t MC_FindAgentByID(MCAgency_t attr, int ID);
 
 /**
  * \brief           Retrieve an agent's Ch interpreter
@@ -382,26 +575,26 @@ extern MCAgent_t MC_FindAgentByID(MCAgency_t attr, int ID);
  * \return          a Ch interpreter of type 'ChInterp_t' on success, or
  *                  NULL on failure.
  */
-extern /*ChInterp_t*/ void* MC_GetAgentExecEngine(MCAgent_t agent);
+EXPORTMC extern /*ChInterp_t*/ void* MC_GetAgentExecEngine(MCAgent_t agent);
 
 /**
  * \brief           Retrieve an agent's id
  */
-extern int MC_GetAgentID(MCAgent_t agent);
+EXPORTMC extern int MC_GetAgentID(MCAgent_t agent);
 
 /*
  * \brief           Retrieve an agent's name
  *
  * \return          a malloc'd character string on success, or NULL on failure
  */
-extern char* MC_GetAgentName(MCAgent_t agent);
+EXPORTMC extern char* MC_GetAgentName(MCAgent_t agent);
 
 /**
  * \brief           Retrive the number of tasks an agent has
  * \section Example
  * \include multi_task_example/client.c
  */
-extern int MC_GetAgentNumTasks(MCAgent_t agent);
+EXPORTMC extern int MC_GetAgentNumTasks(MCAgent_t agent);
 
 /**
  * \brief           Get an agent's return data
@@ -419,7 +612,7 @@ extern int MC_GetAgentNumTasks(MCAgent_t agent);
  * This is the agent which gets the data
  * \include mc_array_return_example/agent.xml
  */
-extern int MC_GetAgentReturnData(
+EXPORTMC extern int MC_GetAgentReturnData(
         MCAgent_t agent,
         int task_num,
         void **data,
@@ -431,21 +624,21 @@ extern int MC_GetAgentReturnData(
  *
  * \return          returns type 'enum MC_AgentStatus_e'
  */
-extern int MC_GetAgentStatus(MCAgent_t agent);
+EXPORTMC extern int MC_GetAgentStatus(MCAgent_t agent);
 
 /**
  * \brief           Get an agent's type
  *
  * \return          returns type 'enum MC_AgentType_e'
  */
-extern enum MC_AgentType_e MC_GetAgentType(MCAgent_t agent);
+EXPORTMC extern enum MC_AgentType_e MC_GetAgentType(MCAgent_t agent);
 
 /**
  * \brief           Get an agent's xml string
  *
  * \return          a malloc'd character string containing the agent's xml code
  */
-extern char* MC_GetAgentXMLString(MCAgent_t agent);
+EXPORTMC extern char* MC_GetAgentXMLString(MCAgent_t agent);
 
 /**
  * \brief           Halt an agency: Do not process new entries in queues.
@@ -454,7 +647,7 @@ extern char* MC_GetAgentXMLString(MCAgent_t agent);
  *
  * \return          0 on success, non-zero on failure.
  */
-extern int MC_HaltAgency(MCAgency_t agency);
+EXPORTMC extern int MC_HaltAgency(MCAgency_t agency);
 
 /**
  * \brief           Initialize and start a MobileC agency
@@ -466,7 +659,7 @@ extern int MC_HaltAgency(MCAgency_t agency);
  * \section Example
  * \include hello_world/server.c
  */
-extern MCAgency_t MC_Initialize(
+EXPORTMC extern MCAgency_t MC_Initialize(
         int port,
         MCAgencyOptions_t *options);
 
@@ -482,14 +675,14 @@ extern MCAgency_t MC_Initialize(
  * \section Example
  * \include hello_world/server.c
  */
-extern int MC_InitializeAgencyOptions(struct MCAgencyOptions_s* options);
+EXPORTMC extern int MC_InitializeAgencyOptions(struct MCAgencyOptions_s* options);
 
 /**
  * \brief           Wait indefinitely
  *
  * \note            This function is intended to block the calling thread forever.
  */
- extern int MC_MainLoop(MCAgency_t attr);
+ EXPORTMC extern int MC_MainLoop(MCAgency_t attr);
 
 /**
  * \brief           Locks a MobileC synchronization variable as a mutex
@@ -505,7 +698,7 @@ extern int MC_InitializeAgencyOptions(struct MCAgencyOptions_s* options);
  * \include agent_mutex_example/sleep.xml
  * \include agent_mutex_example/wake.xml
  */
-extern int MC_MutexLock(MCAgency_t attr, int id); 
+EXPORTMC extern int MC_MutexLock(MCAgency_t attr, int id); 
 
 /*
  * \brief           Unlocks a MobileC synchronization variable
@@ -517,7 +710,7 @@ extern int MC_MutexLock(MCAgency_t attr, int id);
  *
  * \see             MC_MutexLock()
  */
-extern int MC_MutexUnlock(MCAgency_t attr, int id); 
+EXPORTMC extern int MC_MutexUnlock(MCAgency_t attr, int id); 
 
 /**
  * \brief           Register a new service with the Directory Facilitator
@@ -536,7 +729,7 @@ extern int MC_MutexUnlock(MCAgency_t attr, int id);
  * \section Example
  * \include mc_df_service_test/service_provider_1.xml
  */
-extern int MC_RegisterService(
+EXPORTMC extern int MC_RegisterService(
         MCAgency_t agency,
         /* Optional input args: Must input 
          * MCAgent_t OR
@@ -554,14 +747,14 @@ extern int MC_RegisterService(
  *
  * \return          0 on success, non-zero on failure.
  */
-extern int MC_ResumeAgency(MCAgency_t agency);
+EXPORTMC extern int MC_ResumeAgency(MCAgency_t agency);
 
 /**
  * \brief           Retrieves the oldest agent from an agency
  *
  * \return          a valid agent or NULL on failure
  */
-extern MCAgent_t MC_RetrieveAgent(MCAgency_t attr);
+EXPORTMC extern MCAgent_t MC_RetrieveAgent(MCAgency_t attr);
 
 /**
  * \brief           Post to a MobileC synchronization variable semaphore
@@ -574,7 +767,7 @@ extern MCAgent_t MC_RetrieveAgent(MCAgency_t attr);
  * \section Example
  * \include agent_semaphore_example/wake.xml
  */
-extern int MC_SemaphorePost(MCAgency_t attr, int id);
+EXPORTMC extern int MC_SemaphorePost(MCAgency_t attr, int id);
 
 /**
  * \brief           Decreases a MobileC synchronization variable semaphore count by one
@@ -589,7 +782,7 @@ extern int MC_SemaphorePost(MCAgency_t attr, int id);
  * \section Example
  * \include agent_semaphore_example/sleep.xml
  */
-extern int MC_SemaphoreWait(MCAgency_t attr, int id);
+EXPORTMC extern int MC_SemaphoreWait(MCAgency_t attr, int id);
 
 /**
  * \brief           Sets default incoming agent status
@@ -602,7 +795,7 @@ extern int MC_SemaphoreWait(MCAgency_t attr, int id);
  * \note            using this function will override any status the incoming
  *                  agent attempts to set for itself.
  */
-extern int MC_SetDefaultAgentStatus(MCAgency_t agency, enum MC_AgentStatus_e status);
+EXPORTMC extern int MC_SetDefaultAgentStatus(MCAgency_t agency, enum MC_AgentStatus_e status);
 
 /**
  * \brief           Sets a MobileC thread to "on" status.
@@ -617,7 +810,17 @@ extern int MC_SetDefaultAgentStatus(MCAgency_t agency, enum MC_AgentStatus_e sta
  *                  an agency is started with MC_Initialize, the MC_SetThread
  *                  functions will have no effect.
  */
-extern int MC_SetThreadOn(MCAgencyOptions_t *options, enum MC_ThreadIndex_e index);
+EXPORTMC extern int MC_SetThreadOn(MCAgencyOptions_t *options, enum MC_ThreadIndex_e index);
+
+/** 
+ * \brief           Set all Mobile-C threads on.
+ *
+ * \param options   MobileC options structure, initialized with
+ *                  MC_InitializeAgencyOptions()
+ *
+ * \return          0 on success, error code on failure.
+ */
+EXPORTMC extern int MC_SetThreadsAllOn(MCAgencyOptions_t* options);
 
 /**
  * \brief           Sets a MobileC thread to "off" status.
@@ -632,21 +835,31 @@ extern int MC_SetThreadOn(MCAgencyOptions_t *options, enum MC_ThreadIndex_e inde
  *                  an agency is started with MC_Initialize, the MC_SetThread
  *                  functions will have no effect.
  */
-extern int MC_SetThreadOff(MCAgencyOptions_t *options, enum MC_ThreadIndex_e index );
+EXPORTMC extern int MC_SetThreadOff(MCAgencyOptions_t *options, enum MC_ThreadIndex_e index );
+
+/**
+ * \brief           Set all MobileC threads to 'off' status.
+ *
+ * \param options   a MobileC options structure initialized with with the
+ *                  MC_InitializeAgencyOptions() function.
+ *
+ * \return          0 on success, error code on failure.
+ */
+EXPORTMC extern int MC_SetThreadsAllOff(MCAgencyOptions_t* options);
 
 /**
  * \brief           Prints an agents code to stdout
  *
  * \return          0 on success, error_code_t on failure
  */
-extern int MC_PrintAgentCode(MCAgent_t agent);
+EXPORTMC extern int MC_PrintAgentCode(MCAgent_t agent);
 
 /**
  * \brief           Retrieves an agent's Ch code
  *
  * \return          a malloc'd character string on success, NULL on failure
  */
-extern char * MC_RetrieveAgentCode(MCAgent_t agent);
+EXPORTMC extern char * MC_RetrieveAgentCode(MCAgent_t agent);
 
 /**
  * \brief           Reset a MobileC signal
@@ -655,7 +868,7 @@ extern char * MC_RetrieveAgentCode(MCAgent_t agent);
  *
  * \see             MC_WaitSignal()
  */
-extern int MC_ResetSignal(MCAgency_t attr);
+EXPORTMC extern int MC_ResetSignal(MCAgency_t attr);
 
 /**
  * \brief           Search the directory facilitator for a service
@@ -672,7 +885,7 @@ extern int MC_ResetSignal(MCAgency_t attr);
  * \section Example
  * \include mc_df_service_test/test1.xml
  */
-extern int MC_SearchForService(
+EXPORTMC extern int MC_SearchForService(
         /* Input args */
         MCAgency_t attr, 
         const char *searchString,
@@ -690,7 +903,7 @@ extern int MC_SearchForService(
  * \param hostname  host to send the message to
  * \param port      port to send the message to
  */
-extern int MC_SendAgentMigrationMessage(MCAgency_t attr,
+EXPORTMC extern int MC_SendAgentMigrationMessage(MCAgency_t attr,
         const char *message,
         const char *hostname,
         int port);
@@ -703,7 +916,7 @@ extern int MC_SendAgentMigrationMessage(MCAgency_t attr,
  * \param hostname  hostname to send the agent to
  * \param port      port to send the agent to
  */
-extern int MC_SendAgentMigrationMessageFile(MCAgency_t attr, 
+EXPORTMC extern int MC_SendAgentMigrationMessageFile(MCAgency_t attr, 
         const char *filename,
         const char *hostname,
         int port);
@@ -716,7 +929,7 @@ extern int MC_SendAgentMigrationMessageFile(MCAgency_t attr,
  *
  * \return          0 on success, or error_code_t on failure
  */
-extern int MC_SetAgentStatus(MCAgent_t agent, int status);
+EXPORTMC extern int MC_SetAgentStatus(MCAgent_t agent, int status);
 
 /**
  * \brief           Set up a steerable algorithm
@@ -733,7 +946,7 @@ extern int MC_SetAgentStatus(MCAgent_t agent, int status);
  * \section Example
  * \include steer_example/mc_server.c
  */
-extern int MC_Steer( MCAgency_t attr, int (*funcptr)(void* data), void *arg);
+EXPORTMC extern int MC_Steer( MCAgency_t attr, int (*funcptr)(void* data), void *arg);
 
 /**
  * \brief           The MobileC user-algorithm steering function
@@ -743,7 +956,7 @@ extern int MC_Steer( MCAgency_t attr, int (*funcptr)(void* data), void *arg);
  * \note            This function belongs inside a user's steerable algorithm.
  * \see             MC_Steer()
  */
-extern enum MC_SteerCommand_e MC_SteerControl(void);
+EXPORTMC extern enum MC_SteerCommand_e MC_SteerControl(void);
 
 /**
  * \brief           Deletes a previously initialized synchronization variable
@@ -753,7 +966,7 @@ extern enum MC_SteerCommand_e MC_SteerControl(void);
  *
  * \return          0 on success, or error_code_t on failure
  */
-extern int MC_SyncDelete(MCAgency_t attr, int id);
+EXPORTMC extern int MC_SyncDelete(MCAgency_t attr, int id);
 
 /**
  * \brief           Initializes a new MobileC synchronization variable
@@ -770,21 +983,21 @@ extern int MC_SyncDelete(MCAgency_t attr, int id);
  * \see             MC_MutexLock(), MC_MutexUnlock(), MC_CondWait(), MC_CondSignal(),
  *                  MC_CondBroadcast, MC_SemaphorePost(), MC_SemaphoreWait()
  */
-extern int MC_SyncInit(MCAgency_t attr, int id);
+EXPORTMC extern int MC_SyncInit(MCAgency_t attr, int id);
 
 /**
  * \brief           Halt a running agent
  *
  * \return          0 on success, error_code_t on failure
  */
-extern int MC_TerminateAgent(MCAgent_t agent);
+EXPORTMC extern int MC_TerminateAgent(MCAgent_t agent);
 
 /**
  * \brief           Wait indefinitely
  *
  * \note            This function is intended to block the calling thread forever.
  */
-/* extern int MC_Wait(MCAgency_t attr); */
+/* EXPORTMC extern int MC_Wait(MCAgency_t attr); */
 
 /**
  * \brief           Wait for an agent arrival event
@@ -792,7 +1005,7 @@ extern int MC_TerminateAgent(MCAgent_t agent);
  * \note            This function blocks until an agent arrival signal is
  *                  triggered, at which point in unblocks.
  */
-extern int MC_WaitAgent(MCAgency_t attr);
+EXPORTMC extern int MC_WaitAgent(MCAgency_t attr);
 
 /**
  * \brief           Wait and retrieve an agent
@@ -802,7 +1015,7 @@ extern int MC_WaitAgent(MCAgency_t attr);
  * \note            This function blocks until the arrival of an agent. The agent
  *                  is retrieved after it is initialized, but before it is executed.
  */
-extern MCAgent_t MC_WaitRetrieveAgent(MCAgency_t attr);
+EXPORTMC extern MCAgent_t MC_WaitRetrieveAgent(MCAgency_t attr);
 
 /**
  * \brief           Wait for a MobileC signal
@@ -817,7 +1030,7 @@ extern MCAgent_t MC_WaitRetrieveAgent(MCAgency_t attr);
  * \section Example
  * \include multi_task_example/client.c
  */
-extern int MC_WaitSignal(MCAgency_t attr, int signals);
+EXPORTMC extern int MC_WaitSignal(MCAgency_t attr, int signals);
 
 #ifdef __cplusplus
 }
