@@ -39,12 +39,21 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifndef _WIN32
+#include <sys/time.h>
+#else
+#include <time.h>
+#endif
 
 #include "include/libmc.h"
 #include "include/macros.h"
 #include "include/mc_platform.h"
 #include "include/message.h"
 #include "include/data_structures.h"
+#include "include/fipa_acl_envelope.h"
+#include "include/fipa_acl.h"
+#include "include/agent.h"
+#include "include/agent_task.h"
 #ifndef HOST_NAME_MAX
 #define HOST_NAME_MAX 255
 #endif
@@ -55,7 +64,253 @@ mc_platform_p g_mc_platform;
 /* **************************** *
  * Libmc Binary Space Functions *
  * **************************** */
+
 int 
+MC_AclDestroy(struct fipa_acl_message_s* message)
+{
+  return fipa_acl_message_Destroy(message);
+}
+
+EXPORTMC fipa_acl_message_t* 
+MC_AclNew(void) {
+  return fipa_acl_message_New();
+}
+
+EXPORTMC int 
+MC_AclPost(MCAgent_t agent, struct fipa_acl_message_s* message)
+{
+  return agent_mailbox_Post(agent->mailbox, message);
+}
+
+EXPORTMC fipa_acl_message_t*
+MC_AclReply(fipa_acl_message_t* acl_message)
+{
+  return fipa_Reply(acl_message);
+}
+
+EXPORTMC fipa_acl_message_t*
+MC_AclRetrieve(MCAgent_t agent)
+{
+  return agent_mailbox_Retrieve(agent->mailbox);
+}
+
+EXPORTMC int
+MC_AclSend(MCAgency_t attr, fipa_acl_message_t* acl)
+{
+  /* FIXME: We may want to multithread this later for asynchronous
+   * sending to multiple hosts. But for now, we will use a simple
+   * loop. */
+  int i;
+  int err;
+  mtp_http_t* msg;
+  dynstring_t* msg_string;
+  message_p mc_message;
+
+  char* host;
+  int port;
+  char* target;
+  MCAgent_t agent;
+  int num_addresses = 0;
+
+  err = fipa_acl_Compose(&msg_string, acl);
+  if( err ) {
+    fprintf(stderr, "ACL Message Compose Error. %s:%d\n", __FILE__, __LINE__);
+    return err;
+  }
+  for(i = 0; i < acl->receiver->num; i++) {
+    /* The receiver may or may not have an address. If it does, send the
+     * message to that address. If not, assume that the target agent is
+     * local and try to send to that agent. */
+    if (acl->receiver->fipa_agent_identifiers[i]->addresses == NULL) {
+      num_addresses = 0;
+    } else {
+      num_addresses = acl->receiver->fipa_agent_identifiers[i]->addresses->num;
+    }
+    if (num_addresses == 0) {
+      agent = MC_FindAgentByName(
+          attr,
+          acl->receiver->fipa_agent_identifiers[i]->name );
+      if (agent == NULL) {
+        fprintf(stderr, "Could not find local agent:%s. %s:%d\n",
+            acl->receiver->fipa_agent_identifiers[i]->name,
+            __FILE__, __LINE__);
+      }
+      MC_AclPost(agent, acl);
+    } else {
+      msg = mtp_http_New();
+      /* Send to the first address listed */
+      err = http_to_hostport(
+          acl->receiver->fipa_agent_identifiers[i]->addresses->urls[0]->str,
+          &host,
+          &port,
+          &target );
+      if (err) {
+        fprintf(stderr, "Invalid address. %s:%d\n", __FILE__, __LINE__);
+        return err;
+      }
+      msg->host = strdup(host);
+      msg->target = strdup(target);
+      msg->message_parts = 2;
+      msg->content = (mtp_http_content_t *)malloc(
+          sizeof(mtp_http_content_t) * 2);
+
+      /* Set up the message envelope */
+      msg->content[0].data = (void*)fipa_envelope_Compose(acl);
+      
+      msg->content[0].content_type = strdup("application/xml");
+
+      /* Set up the ACL message */
+      msg->content[1].data = (void*)strdup(msg_string->message);
+      msg->content[1].content_type = strdup("application/text");
+
+      mc_message = mtp_http_CreateMessage(
+          msg,
+          host,
+          port );
+      mc_message->message_type = FIPA_ACL;
+      mc_message->target = strdup("acc");
+
+      message_Send
+        (
+         mc_message
+        );
+      message_Destroy(mc_message);
+      mtp_http_Destroy(msg);
+      free(host);
+      free(target);
+    }
+  }
+  dynstring_Destroy(msg_string);
+  return 0;
+}
+
+EXPORTMC fipa_acl_message_t* 
+MC_AclWaitRetrieve(MCAgent_t agent)
+{
+  return agent_mailbox_WaitRetrieve(agent->mailbox);
+}
+
+/* ACL Helper Functions Here */
+
+int MC_AclSetPerformative(
+    fipa_acl_message_t* acl,
+    enum fipa_performative_e performative )
+{
+  acl->performative = performative;
+  return 0;
+}
+
+int MC_AclSetSender(
+    fipa_acl_message_t* acl,
+    const char* name,
+    const char* address )
+{
+  if(acl->sender != NULL) {
+    /* There is already a sender. Lets over-write it */
+    fipa_agent_identifier_Destroy(acl->sender);
+  }
+  acl->sender = fipa_agent_identifier_New();
+  acl->sender->name = strdup(name);
+  if (address != NULL) {
+    acl->sender->addresses = fipa_url_sequence_New();
+    acl->sender->addresses->num = 1;
+    acl->sender->addresses->urls = (struct fipa_url_s**)malloc(
+        sizeof(struct fipa_url_s*));
+    acl->sender->addresses->urls[0] = fipa_url_New();
+    acl->sender->addresses->urls[0]->str = strdup(address);
+  }
+
+  return 0;
+}
+  
+int MC_AclAddReceiver(
+    fipa_acl_message_t* acl,
+    const char* name,
+    const char* address )
+{
+  int i;
+  struct fipa_agent_identifier_s** tmp;
+  if (acl->receiver == NULL) {
+    acl->receiver = fipa_agent_identifier_set_New();
+  }
+  acl->receiver_num++;
+
+  acl->receiver->num++;
+  tmp = (struct fipa_agent_identifier_s**)malloc(
+      sizeof(struct fipa_agent_identifier_s*)
+      * acl->receiver->num);
+  /* Copy existing addresses to new array */
+  for(i = 0; i < acl->receiver->num-1; i++) {
+    tmp[i] = acl->receiver->fipa_agent_identifiers[i];
+  }
+  /* Create new receiver */
+  tmp[i] = fipa_agent_identifier_New();
+  tmp[i]->name = strdup(name);
+  if(address != NULL) {
+    tmp[i]->addresses = fipa_url_sequence_New();
+    tmp[i]->addresses->num = 1;
+    tmp[i]->addresses->urls = (struct fipa_url_s**)malloc(
+        sizeof(struct fipa_url_s*));
+    tmp[i]->addresses->urls[0] = fipa_url_New();
+    tmp[i]->addresses->urls[0]->str = strdup(address);
+  }
+  free(acl->receiver->fipa_agent_identifiers);
+  acl->receiver->fipa_agent_identifiers = tmp;
+  return 0;
+}
+
+int MC_AclAddReplyTo(
+    fipa_acl_message_t* acl,
+    const char* name,
+    const char* address)
+{
+  int i;
+  struct fipa_agent_identifier_s** tmp;
+  if (acl->reply_to == NULL) {
+    acl->reply_to = fipa_agent_identifier_set_New();
+  }
+
+  acl->reply_to->num++;
+  tmp = (struct fipa_agent_identifier_s**)malloc(
+      sizeof(struct fipa_agent_identifier_s*)
+      * acl->reply_to->num);
+  /* Copy existing addresses to new array */
+  for(i = 0; i < acl->reply_to->num-1; i++) {
+    tmp[i] = acl->reply_to->fipa_agent_identifiers[i];
+  }
+  /* Create new reply_to */
+  tmp[i] = fipa_agent_identifier_New();
+  tmp[i]->name = strdup(name);
+  if(address != NULL) {
+    tmp[i]->addresses = fipa_url_sequence_New();
+    tmp[i]->addresses->num = 1;
+    tmp[i]->addresses->urls = (struct fipa_url_s**)malloc(
+        sizeof(struct fipa_url_s*));
+    tmp[i]->addresses->urls[0] = fipa_url_New();
+    tmp[i]->addresses->urls[0]->str = strdup(address);
+  }
+  free (acl->reply_to->fipa_agent_identifiers);
+  acl->reply_to->fipa_agent_identifiers = tmp;
+  return 0;
+}
+
+int MC_AclSetContent(
+    fipa_acl_message_t* acl,
+    const char* content )
+{
+  if (acl->content != NULL) {
+    /* There is already content. Lets over-write it. */
+    fipa_string_Destroy(acl->content);
+  }
+  acl->content = fipa_string_New();
+  acl->content->content = strdup(content);
+
+  return 0;
+}
+
+/* End ACL Helper Functions */
+
+EXPORTMC int 
 MC_AddAgent(MCAgency_t attr, MCAgent_t agent) /*{{{*/
 {
   agent->mc_platform = attr->mc_platform;
@@ -95,7 +350,7 @@ MC_Barrier(MCAgency_t attr, int id) /*{{{*/
     return MC_SUCCESS;
 } /*}}}*/
 
-int
+EXPORTMC int
 MC_BarrierInit(MCAgency_t attr, int id, int num_procs) /*{{{*/
 {
     barrier_node_p node;
@@ -109,31 +364,31 @@ MC_BarrierInit(MCAgency_t attr, int id, int num_procs) /*{{{*/
     return MC_SUCCESS;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_BarrierDelete(MCAgency_t attr, int id) /*{{{*/
 {
   return barrier_queue_Delete(id, attr->mc_platform->barrier_queue);
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_CallAgentFunc( /*{{{*/
         MCAgent_t agent,
         const char* funcName,
-        void* returnVal,
+        void* returnVal, /* FIXME: Should this be a double pointer? */
         void* varg)
 {
     int return_code;
-    MUTEX_LOCK(agent->run_lock);
+    MUTEX_LOCK(agent->run_lock); 
     return_code = Ch_CallFuncByName(
             agent->agent_interp,
             funcName,
             returnVal,
             varg);
-    MUTEX_UNLOCK(agent->run_lock);
+    MUTEX_UNLOCK(agent->run_lock); 
     return return_code;
 } /*}}}*/ 
 
-int 
+EXPORTMC int 
 MC_ChInitializeOptions(MCAgency_t attr, ChOptions_t *options) { /*{{{*/
     if(attr->mc_platform == NULL) {
         fprintf(stderr, "MC_ChInitializeOptions must be called after MC_Start()\n");
@@ -151,7 +406,77 @@ MC_ChInitializeOptions(MCAgency_t attr, ChOptions_t *options) { /*{{{*/
     }
 } /*}}}*/
 
-int 
+MCAgent_t
+MC_ComposeAgent(
+    const char* name,
+    const char* home,
+    const char* owner, 
+    const char* code,
+    const char* return_var_name,
+    const char* server,
+    int persistent
+    )
+{
+  agent_p agent;
+  agent = agent_New();
+  if (agent == NULL) return NULL;
+  agent->name = strdup(name);
+  agent->home = strdup(home);
+  agent->owner = strdup(owner);
+
+  agent->orphan = 1;
+
+  agent->agent_type = MC_LOCAL_AGENT;
+  agent->agent_status = MC_WAIT_MESSGSEND;
+
+  agent->datastate = agent_datastate_New();
+  agent->datastate->number_of_tasks = 1;
+  agent->datastate->persistent = persistent;
+  agent->datastate->agent_code_ids = (char**)malloc(
+      sizeof(char*)*2);
+  if(agent->datastate->agent_code_ids == NULL) {
+    fprintf(stderr, "Memory Error %s:%d\n", __FILE__, __LINE__);
+  }
+  agent->datastate->agent_code_ids[0] = strdup("");
+  if(agent->datastate->agent_code_ids[0] == NULL) {
+    fprintf(stderr, "Memory Error %s:%d\n", __FILE__, __LINE__);
+  }
+  agent->datastate->agent_code_ids[1] = NULL;
+
+  agent->datastate->agent_codes = (char**)malloc(
+      sizeof(char*)*2);
+  if(agent->datastate->agent_codes == NULL) {
+    fprintf(stderr, "Memory Error %s:%d\n", __FILE__, __LINE__);
+  }
+  agent->datastate->agent_codes[0] = strdup(code);
+  if(agent->datastate->agent_codes[0] == NULL) {
+    fprintf(stderr, "Memory Error %s:%d\n", __FILE__, __LINE__);
+  }
+  agent->datastate->agent_codes[1] = NULL;
+
+  agent->datastate->agent_code = agent->datastate->agent_codes[0];
+
+  agent->datastate->tasks = (agent_task_t**)malloc(
+      sizeof(agent_task_t*));
+  agent->datastate->tasks[0] = agent_task_New();
+  if(return_var_name == NULL) {
+    agent->datastate->tasks[0]->var_name = strdup("no-return");
+  } else {
+    agent->datastate->tasks[0]->var_name = strdup(return_var_name);
+  }
+  if(agent->datastate->tasks[0]->var_name == NULL) {
+    fprintf(stderr, "Memory Error %s:%d\n", __FILE__, __LINE__);
+  }
+
+  agent->datastate->tasks[0]->server_name = strdup(server);
+  if(agent->datastate->tasks[0]->server_name == NULL) {
+    fprintf(stderr, "Memory Error %s:%d\n", __FILE__, __LINE__);
+  }
+  
+  return agent;
+}
+
+EXPORTMC int 
 MC_CondBroadcast(MCAgency_t attr, int id) /*{{{*/
 {
     syncListNode_t *condnode;
@@ -166,7 +491,7 @@ MC_CondBroadcast(MCAgency_t attr, int id) /*{{{*/
     return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_CondSignal(MCAgency_t attr, int id) /*{{{*/
 {
     syncListNode_t *condnode;
@@ -181,7 +506,7 @@ MC_CondSignal(MCAgency_t attr, int id) /*{{{*/
     return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_CondWait(MCAgency_t attr, int id)  /*{{{*/
 {
     syncListNode_t *condnode;
@@ -203,7 +528,7 @@ MC_CondWait(MCAgency_t attr, int id)  /*{{{*/
     return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_CondReset(MCAgency_t attr, int id) /*{{{*/
 {
     syncListNode_t *condnode;
@@ -228,7 +553,7 @@ MC_CopyAgent(MCAgent_t* agent_out, const MCAgent_t agent_in) /*{{{*/
   return MC_SUCCESS;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_DeleteAgent(MCAgent_t agent) /*{{{*/
 {
     /* Error Checking */
@@ -243,6 +568,25 @@ MC_DeleteAgent(MCAgent_t agent) /*{{{*/
     return MC_SUCCESS;
 } /*}}}*/
 
+int MC_DestroyServiceSearchResult(
+    char** agentName,
+    char** serviceName,
+    int* agentID,
+    int numResult)
+{
+  int i;
+  for(i = 0;i < numResult; i++)
+  {
+    free(agentName[i]);
+    free(serviceName[i]);
+  }
+  free(agentName);
+  free(serviceName);
+  free(agentID);
+
+  return 0;
+}
+
 int 
 MC_DeregisterService( /*{{{*/
         MCAgency_t agency,
@@ -272,7 +616,7 @@ MC_DeregisterService( /*{{{*/
     return err_code;
 } /*}}}*/
 
-int
+EXPORTMC int
 MC_End(MCAgency_t agency) /*{{{*/
 {
   /* Now, we must stop all the running pthreads somehow... */
@@ -313,7 +657,7 @@ MC_End(MCAgency_t agency) /*{{{*/
   return 0;
 } /*}}}*/
 
-MCAgent_t 
+EXPORTMC MCAgent_t 
 MC_FindAgentByName( MCAgency_t attr, /*{{{*/
     const char *name) 
 {
@@ -326,7 +670,7 @@ MC_FindAgentByName( MCAgency_t attr, /*{{{*/
   }
 } /*}}}*/
 
-MCAgent_t
+EXPORTMC MCAgent_t
 MC_FindAgentByID( MCAgency_t attr, /*{{{*/
     int ID)
 {
@@ -358,7 +702,7 @@ MC_GetAgentArrivalTime(MCAgent_t agent) /*{{{*/
   }
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_GetAgentStatus(MCAgent_t agent) /*{{{*/
 {
   int status;
@@ -368,7 +712,7 @@ MC_GetAgentStatus(MCAgent_t agent) /*{{{*/
   return status;
 } /*}}}*/
 
-char*
+EXPORTMC char*
 MC_GetAgentXMLString(MCAgent_t agent) /*{{{*/
 {
   char *ret;
@@ -380,13 +724,13 @@ MC_GetAgentXMLString(MCAgent_t agent) /*{{{*/
 } /*}}}*/
 
 /*ChInterp_t */
-void*
+EXPORTMC void*
 MC_GetAgentExecEngine(MCAgent_t agent) /*{{{*/
 {
   return agent->agent_interp;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_GetAgentID( /*{{{*/
     MCAgent_t agent
     )
@@ -394,7 +738,7 @@ MC_GetAgentID( /*{{{*/
   return agent->id;
 } /*}}}*/
 
-char* 
+EXPORTMC char* 
 MC_GetAgentName( /*{{{*/
     MCAgent_t agent
     )
@@ -412,7 +756,7 @@ MC_GetAgentName( /*{{{*/
   return name;
 } /*}}}*/
 
-int
+EXPORTMC int
 MC_GetAgentReturnData( /*{{{*/
     MCAgent_t agent,
     int task_num,
@@ -475,13 +819,13 @@ MC_GetAgentReturnData( /*{{{*/
   return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_GetAgentNumTasks(MCAgent_t agent) /*{{{*/
 {
   return agent->datastate->number_of_tasks;
 } /*}}}*/
 
-enum MC_AgentType_e
+EXPORTMC enum MC_AgentType_e
 MC_GetAgentType(MCAgent_t agent) /*{{{*/
 {
   if (agent != NULL) {
@@ -505,7 +849,7 @@ MC_GetAllAgents(MCAgency_t attr, MCAgent_t **agents, int* num_agents) /*{{{*/
   while (agent_queue_SearchIndex(attr->mc_platform->agent_queue, index) != NULL) {
     index++;
   }
-  *agents = (MCAgent_t **)malloc(sizeof(MCAgent_t*) * index);
+  *agents = (MCAgent_t *)malloc(sizeof(MCAgent_t*) * index);
   *num_agents = index;
   /* Assign the agents */
   index = 0;
@@ -527,7 +871,7 @@ MC_GetAllAgents(MCAgency_t attr, MCAgent_t **agents, int* num_agents) /*{{{*/
 
 
 
-int
+EXPORTMC int
 MC_HaltAgency(MCAgency_t attr) /*{{{*/
 {
   MUTEX_LOCK(attr->mc_platform->giant_lock);
@@ -536,7 +880,7 @@ MC_HaltAgency(MCAgency_t attr) /*{{{*/
   return 0;
 } /*}}}*/
 
-MCAgency_t 
+EXPORTMC MCAgency_t 
 MC_Initialize( /*{{{*/
     int port,
     MCAgencyOptions_t *options)
@@ -591,7 +935,7 @@ MC_Initialize( /*{{{*/
   return ret;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_InitializeAgencyOptions(struct MCAgencyOptions_s* options) /*{{{*/
 {
   int i;
@@ -608,7 +952,7 @@ MC_InitializeAgencyOptions(struct MCAgencyOptions_s* options) /*{{{*/
   return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_MutexLock(MCAgency_t attr, int id) /*{{{*/
 {
   syncListNode_t *syncnode;
@@ -620,7 +964,7 @@ MC_MutexLock(MCAgency_t attr, int id) /*{{{*/
   return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_MutexUnlock(MCAgency_t attr, int id) /*{{{*/
 {
   syncListNode_t *syncnode;
@@ -632,7 +976,7 @@ MC_MutexUnlock(MCAgency_t attr, int id) /*{{{*/
   return 0;
 } /*}}}*/
 
-int
+EXPORTMC int
 MC_PrintAgentCode(MCAgent_t agent) /*{{{*/
 {
   int progress;
@@ -648,7 +992,7 @@ MC_PrintAgentCode(MCAgent_t agent) /*{{{*/
   return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_RegisterService( /*{{{*/
     MCAgency_t agency,
     MCAgent_t agent,
@@ -727,7 +1071,7 @@ MC_RegisterService( /*{{{*/
       );
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_ResumeAgency(MCAgency_t attr) /*{{{*/
 {
   MUTEX_LOCK(attr->mc_platform->giant_lock);
@@ -736,7 +1080,7 @@ MC_ResumeAgency(MCAgency_t attr) /*{{{*/
   return 0;
 } /*}}}*/
 
-MCAgent_t
+EXPORTMC MCAgent_t
 MC_RetrieveAgent(MCAgency_t attr) /*{{{*/
   /* This function retrieves the first agent with agent_status
      MC_AGENT_NEUTRAL it finds. If there are no agents with
@@ -766,7 +1110,7 @@ MC_RetrieveAgent(MCAgency_t attr) /*{{{*/
   return ret;
 }/*}}}*/
 
-char * 
+EXPORTMC char * 
 MC_RetrieveAgentCode(MCAgent_t agent) /*{{{*/
 {
   char *buf;
@@ -782,7 +1126,7 @@ MC_RetrieveAgentCode(MCAgent_t agent) /*{{{*/
   return buf;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_ResetSignal(MCAgency_t attr) /*{{{*/
 {
   MUTEX_LOCK(attr->mc_platform->giant_lock);
@@ -793,7 +1137,7 @@ MC_ResetSignal(MCAgency_t attr) /*{{{*/
   return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_SearchForService( /*{{{*/
     /* Input args */
     MCAgency_t attr, 
@@ -843,7 +1187,7 @@ MC_SearchForService( /*{{{*/
   return MC_SUCCESS;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_SemaphorePost(MCAgency_t attr, int id) /*{{{*/
 {
   syncListNode_t *syncnode;
@@ -855,7 +1199,7 @@ MC_SemaphorePost(MCAgency_t attr, int id) /*{{{*/
   return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_SemaphoreWait(MCAgency_t attr, int id) /*{{{*/
 {
   syncListNode_t *syncnode;
@@ -877,7 +1221,7 @@ MC_SendCh(MCAgency_t attr, /*{{{*/
   return -1;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_SendAgentMigrationMessage(MCAgency_t attr, /*{{{*/
     const char *string,
     const char *hostname,
@@ -892,7 +1236,8 @@ MC_SendAgentMigrationMessage(MCAgency_t attr, /*{{{*/
        message,
        string,
        hostname,
-       port
+       port,
+       "ams"
       )
     )
   {
@@ -907,7 +1252,7 @@ MC_SendAgentMigrationMessage(MCAgency_t attr, /*{{{*/
   }
 } /*}}}*/
 
-int
+EXPORTMC int
 MC_SendAgentMigrationMessageFile(MCAgency_t attr,  /*{{{*/
     const char *filename, 
     const char *hostname,
@@ -943,7 +1288,8 @@ MC_SendAgentMigrationMessageFile(MCAgency_t attr,  /*{{{*/
          message,
          buf,
          hostname,
-         port
+         port,
+         "ams"
         )
       )
     {
@@ -964,7 +1310,8 @@ MC_SendAgentMigrationMessageFile(MCAgency_t attr,  /*{{{*/
          message,
          buf,
          hostname,
-         port
+         port,
+         "ams"
         )
       )
     {
@@ -981,7 +1328,7 @@ MC_SendAgentMigrationMessageFile(MCAgency_t attr,  /*{{{*/
   return ret;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_SendSteerCommand(MCAgency_t attr, enum MC_SteerCommand_e cmd) /*{{{*/
 {
   MUTEX_LOCK(attr->mc_platform->MC_steer_lock);
@@ -1016,21 +1363,41 @@ MC_SetDefaultAgentStatus(/*{{{*/
   return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_SetThreadOn(MCAgencyOptions_t *options, enum MC_ThreadIndex_e index) /*{{{*/
 {
   SET_THREAD_ON(options->threads, index);
   return 0;
 } /*}}}*/
 
-int
+EXPORTMC int 
+MC_SetThreadsAllOn(MCAgencyOptions_t* options)
+{
+  int i;
+  for(i = 0; i < MC_THREAD_ALL; i++) {
+    SET_THREAD_ON(options->threads, i);
+  }
+  return 0;
+}
+
+EXPORTMC int
 MC_SetThreadOff(MCAgencyOptions_t *options, enum MC_ThreadIndex_e index) /*{{{*/
 {
   SET_THREAD_OFF(options->threads, index);
   return 0;
 } /*}}}*/
 
-int 
+EXPORTMC int
+MC_SetThreadsAllOff(MCAgencyOptions_t* options)
+{
+  int i;
+  for(i = 0; i < MC_THREAD_ALL; i++) {
+    SET_THREAD_OFF(options->threads, i);
+  }
+  return 0;
+}
+
+EXPORTMC int 
 MC_Steer(                        /*{{{*/
     MCAgency_t attr,
     int (*funcptr)(void* data),
@@ -1049,7 +1416,7 @@ MC_Steer(                        /*{{{*/
   return 0;
 } /*}}}*/
 
-enum MC_SteerCommand_e 
+EXPORTMC enum MC_SteerCommand_e 
 MC_SteerControl(void) /*{{{*/
 {
   extern mc_platform_p g_mc_platform;
@@ -1065,7 +1432,7 @@ MC_SteerControl(void) /*{{{*/
   return g_mc_platform->MC_steer_command;
 } /*}}}*/
 
-int 
+EXPORTMC int 
 MC_SyncDelete(MCAgency_t attr, int id) /*{{{*/
 {
   syncListNode_t *sync_node;
@@ -1095,7 +1462,7 @@ MC_SyncDelete(MCAgency_t attr, int id) /*{{{*/
   return syncListNodeDestroy(sync_node);
 } /*}}}*/
 
-int
+EXPORTMC int
 MC_SyncInit(MCAgency_t attr, int id) /*{{{*/
 {
   syncListNode_t *node;
@@ -1120,17 +1487,19 @@ MC_SyncInit(MCAgency_t attr, int id) /*{{{*/
   return id;
 }/*}}}*/
 
-int 
+EXPORTMC int 
 MC_TerminateAgent(MCAgent_t agent) /*{{{*/
 {
-  int status;
-  status = Ch_Abort (agent->agent_interp);
+  int status=0;
+  if(agent->agent_interp != NULL) {
+    status = Ch_Abort (agent->agent_interp);
+  }
   return status;
 } /*}}}*/
 
 
 #ifdef _WIN32
-BOOL
+EXPORTMC BOOL
 MC_MainLoop(MCAgency_t attr) /*{{{*/
 {
   /*    return CloseHandle(attr->global->gaf_ap->thread_id[attr->global->mr_index]); */
@@ -1145,7 +1514,7 @@ MC_MainLoop(MCAgency_t attr)
 } /*}}}*/
 #endif
 
-int
+EXPORTMC int
 MC_WaitAgent(MCAgency_t attr) /*{{{*/
 {
   int size;
@@ -1165,7 +1534,7 @@ MC_WaitAgent(MCAgency_t attr) /*{{{*/
   return 0;
 } /*}}}*/
 
-MCAgent_t
+EXPORTMC MCAgent_t
 MC_WaitRetrieveAgent(MCAgency_t attr) /*{{{*/
 {
   int index;
@@ -1183,7 +1552,7 @@ MC_WaitRetrieveAgent(MCAgency_t attr) /*{{{*/
 /* This function blocks until one of the signals in argument
  * 'signals' is signalled. 'signals' must be a binary | combination
  * of enum MC_Signal_e type. */
-int
+EXPORTMC int
 MC_WaitSignal(MCAgency_t attr, int signals) /*{{{*/
 {
   MUTEX_LOCK(attr->mc_platform->MC_signal_lock);
@@ -1204,6 +1573,218 @@ MC_WaitSignal(MCAgency_t attr, int signals) /*{{{*/
  * Ch Space chdl functions *
  * *********************** */
 
+/* MC_AclDestroy */
+int MC_AclDestroy_chdl(void* varg)
+{
+  int retval;
+  fipa_acl_message_t* acl_message;
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+  acl_message = Ch_VaArg(interp, ap, fipa_acl_message_t*);
+  retval = MC_AclDestroy(acl_message);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* MC_AclNew */
+void* MC_AclNew_chdl(void* varg)
+{
+  void* retval;
+  retval = (void*)MC_AclNew();
+  return retval;
+}
+
+/* MC_AclPost */
+int MC_AclPost_chdl(void* varg)
+{
+  int retval;
+  agent_p agent;
+  fipa_acl_message_t* acl_message;
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+  agent = Ch_VaArg(interp, ap, agent_p);
+  acl_message = Ch_VaArg(interp, ap, fipa_acl_message_t*);
+  retval = MC_AclPost(agent, acl_message);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* MC_AclReply */
+EXPORTCH void*
+MC_AclReply_chdl(void* varg)
+{
+  void* retval;
+  fipa_acl_message_t* acl_message;
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+  acl_message = Ch_VaArg(interp, ap, fipa_acl_message_t*);
+  retval = (void*)MC_AclReply(acl_message);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* MC_AclRetrieve */
+EXPORTCH void*
+MC_AclRetrieve_chdl(void* varg)
+{
+  void* retval;
+  MCAgent_t agent;
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+  agent = Ch_VaArg(interp, ap, MCAgent_t);
+  retval = MC_AclRetrieve(agent);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* MC_AclSend */
+EXPORTCH int
+MC_AclSend_chdl(void* varg)
+{
+  int retval;
+  fipa_acl_message_t* acl_message;
+  MCAgency_t temp_attr;
+  extern mc_platform_p g_mc_platform;
+
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  temp_attr = (MCAgency_t)malloc(sizeof(struct agency_s));
+  CHECK_NULL(temp_attr, exit(-1));
+  temp_attr->mc_platform = g_mc_platform;
+
+  Ch_VaStart(interp, ap, varg);
+  acl_message = (fipa_acl_message_t*) Ch_VaArg(interp, ap, void*);
+  retval = MC_AclSend(temp_attr, acl_message);
+  Ch_VaEnd(interp, ap);
+  free(temp_attr);
+  return retval;
+}
+
+/* MC_AclWaitRetrieve */
+EXPORTCH void*
+MC_AclWaitRetrieve_chdl(void *varg)
+{
+  void* retval;
+  MCAgent_t agent;
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+  agent = Ch_VaArg(interp, ap, MCAgent_t);
+  retval = MC_AclWaitRetrieve(agent);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* BEGIN Acl helper functions */
+
+/* MC_AclSetPerformative */
+EXPORTCH int
+MC_AclSetPerformative_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  enum fipa_performative_e performative;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  performative = Ch_VaArg(interp, ap, enum fipa_performative_e);
+  retval = MC_AclSetPerformative(acl, performative);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* MC_AclSetSender */
+EXPORTCH int
+MC_AclSetSender_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  char* name;
+  char* address;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  name = Ch_VaArg(interp, ap, char*);
+  address = Ch_VaArg(interp, ap, char*);
+  retval = MC_AclSetSender(acl, name, address);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* MC_AclAddReceiver */
+EXPORTCH int
+MC_AclAddReceiver_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  char* name;
+  char* address;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  name = Ch_VaArg(interp, ap, char*);
+  address = Ch_VaArg(interp, ap, char*);
+  retval = MC_AclAddReceiver(acl, name, address);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* MC_AclAddReplyTo */
+EXPORTCH int
+MC_AclAddReplyTo_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  char* name;
+  char* address;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  name = Ch_VaArg(interp, ap, char*);
+  address = Ch_VaArg(interp, ap, char*);
+  retval = MC_AclAddReplyTo(acl, name, address);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* MC_AclSetContent */
+EXPORTCH int
+MC_AclSetContent_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  char* content;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  content = Ch_VaArg(interp, ap, char*);
+  retval = MC_AclSetContent(acl, content);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
+/* END Acl Helper Functions */
+
 /* MC_AddAgent */
 EXPORTCH int
 MC_AddAgent_chdl(void *varg) /*{{{*/
@@ -1352,6 +1933,46 @@ MC_CondBroadcast_chdl(void *varg) /*{{{*/
   return retval;
 } /*}}}*/
 
+/* MC_ComposeAgent */
+EXPORTCH MCAgent_t 
+MC_ComposeAgent_chdl(void *varg) /*{{{*/
+{
+  MCAgent_t retval;
+
+  /* Function Args */
+  const char* name;
+  const char* home;
+  const char* owner;  
+  const char* code;
+  const char* return_var_name;
+  const char* server;
+  int persistent;
+
+  ChInterp_t interp;
+  ChVaList_t ap;
+
+  Ch_VaStart(interp, ap, varg);
+
+  name  = Ch_VaArg(interp, ap, const char*);
+  home  = Ch_VaArg(interp, ap, const char*);
+  owner = Ch_VaArg(interp, ap, const char*);
+  code = Ch_VaArg(interp, ap, const char*);
+  return_var_name = Ch_VaArg(interp, ap, const char*);
+  server = Ch_VaArg(interp, ap, const char*);
+  persistent = Ch_VaArg(interp, ap, int);
+
+  retval= MC_ComposeAgent(
+      name,
+      home,
+      owner,
+      code,
+      return_var_name,
+      server,
+      persistent);
+  Ch_VaEnd(interp, ap);
+  return retval;
+} /*}}}*/
+
 /* MC_CondSignal_chdl*/
 EXPORTCH int
 MC_CondSignal_chdl(void *varg) /*{{{*/
@@ -1436,6 +2057,32 @@ MC_DeleteAgent_chdl(void *varg) /*{{{*/
   return retval;
 } /*}}}*/
 
+EXPORTCH int
+MC_DestroyServiceSearchResult_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  char** agentName;
+  char** serviceName;
+  int* agentID;
+  int numResult;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  agentName = Ch_VaArg(interp, ap, char**);
+  serviceName = Ch_VaArg(interp, ap, char**);
+  agentID = Ch_VaArg(interp, ap, int*);
+  numResult = Ch_VaArg(interp, ap, int);
+
+  retval = MC_DestroyServiceSearchResult(
+      agentName,
+      serviceName,
+      agentID,
+      numResult );
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
 /* MC_DeregisterService_chdl */
 EXPORTCH int
 MC_DeregisterService_chdl(void *varg) /*{{{*/
@@ -1909,12 +2556,15 @@ EXPORTCH int
 MC_SendAgentMigrationMessage_chdl(void *varg) /*{{{*/
 {
   MCAgency_t temp_attr;
+  extern mc_platform_p g_mc_platform;
   const char *message, *hostname;
   int port, retval;
   ChInterp_t interp;
   ChVaList_t ap;
 
-  temp_attr = NULL;
+  temp_attr = (MCAgency_t)malloc(sizeof(struct agency_s));
+  CHECK_NULL(temp_attr, exit(-1));
+  temp_attr->mc_platform = g_mc_platform;
 
   Ch_VaStart(interp, ap, varg);
   message = Ch_VaArg(interp, ap, char *);
@@ -1922,6 +2572,7 @@ MC_SendAgentMigrationMessage_chdl(void *varg) /*{{{*/
   port = Ch_VaArg(interp, ap, int);
   retval = MC_SendAgentMigrationMessage(temp_attr, message, hostname, port);
   Ch_VaEnd(interp, ap);
+  free(temp_attr);
   return retval;
 } /*}}}*/
 
