@@ -54,6 +54,12 @@ asm_Initialize(mc_platform_p mc_platform)
   mc_asm = (mc_asm_p)malloc(sizeof(mc_asm_t));
   CHECK_NULL(mc_asm, exit(0););
 
+  mc_asm->waiting = 0;
+  mc_asm->waiting_lock = (MUTEX_T*)malloc(sizeof(MUTEX_T));
+  MUTEX_INIT(mc_asm->waiting_lock);
+  mc_asm->waiting_cond = (COND_T*)malloc(sizeof(COND_T));
+  COND_INIT(mc_ams->waiting_cond);
+
   mc_asm->mc_platform = mc_platform;
   mc_asm->home_encryption_info = (asm_node_p)malloc(sizeof(asm_node_t));
   
@@ -194,6 +200,12 @@ asm_Thread(LPVOID arg)
        (mc_platform->asm_queue->size == old_asm_queue_size)
       ) 
       {
+        /* Set waiting flag */
+        MUTEX_LOCK(mc_platform->security_manager->waiting_lock);
+        mc_platform->security_manager->waiting = 1;
+        COND_BROADCAST(mc_platform->security_manager->waiting_cond);
+        MUTEX_UNLOCK(mc_platform->security_manager->waiting_lock);
+        /* Wait for activity */
         COND_WAIT
           (
            mc_platform->asm_message_queue->cond,
